--
-- Extended Content Support
--

--
-- Additional columns
-- Segment Type
alter table if exists segment_type
    add column if not exists content_type smallint not null default 0;
alter table if exists segment_type
    add column if not exists format smallint not null default 0;
alter table if exists segment_type
    add column if not exists storage smallint not null default 0;
alter table if exists segment_type
    add column if not exists distributions varchar(255) not null default 'UNIVERSAL';
alter table if exists segment_type
    add column if not exists architectures varchar(255) not null default 'UNIVERSAL';

-- Segment Data
alter table if exists segment_data
    add column if not exists distribution smallint not null default 0;
alter table if exists segment_data
    add column if not exists architecture smallint not null default 0;
alter table if exists segment_data
    add column if not exists patch bigint not null default 0;
alter table if exists segment_data
    add column if not exists build bigint not null default 0;
alter table if exists segment_data
    add column if not exists url text default null;
alter table if exists segment_data
    add column if not exists content_hash varchar(64) default null;
alter table if exists segment_data
    add column if not exists content_size bigint not null default (-1);

alter table if exists segment_data alter column body drop not null;

-- Assignment
alter table if exists assignment
    add column if not exists distribution smallint not null default 0;
alter table if exists assignment
    add column if not exists architecture smallint not null default 0;
alter table if exists assignment
    add column if not exists version_patch bigint not null default (-1);
alter table if exists assignment
    add column if not exists version_build bigint not null default (-1);

-- Segment Version
alter table if exists segment_version
    add column if not exists distribution smallint not null default 0;
alter table if exists segment_version
    add column if not exists architecture smallint not null default 0;
alter table if exists segment_version
    add column if not exists version_patch bigint not null default (-1);
alter table if exists segment_version
    add column if not exists version_build bigint not null default (-1);
alter table if exists segment_version
    add column if not exists effective_patch bigint not null default 0;
alter table if exists segment_version
    add column if not exists effective_build bigint not null default 0;
alter table if exists segment_version
    add column if not exists url text default null;
alter table if exists segment_version
    add column if not exists content_hash varchar(64) default null;
alter table if exists segment_version
    add column if not exists content_size bigint not null default (-1);


--
-- Table Constraints

-- Drop table old constraints
alter table if exists segment_data
    drop constraint if exists segment_data_segment_id_platform_major_minor_key;
alter table if exists assignment
    drop constraint if exists assignment_assignment_scope_urn_segment_id_platform_key;


-- Add new table constraints
do
$do$
    begin
        -- Segment Data
        if not exists(select constraint_name
                      from information_schema.table_constraints
                      where table_name = 'segment_data'
                        and constraint_name = 'segment_data_unique_entries_key')
        then
            alter table if exists segment_data
                add constraint segment_data_unique_entries_key
                    UNIQUE (segment_id, platform, distribution, architecture, major, minor, patch, build);
        end if;

        -- Assignment
        if not exists(select constraint_name
                      from information_schema.table_constraints
                      where table_name = 'assignment'
                        and constraint_name = 'assignment_unique_entries_key')
        then
            alter table if exists assignment
                add constraint assignment_unique_entries_key
                    UNIQUE (assignment_scope_urn, segment_id, platform, distribution, architecture);
        end if;
    end;
$do$;


--
-- Segment Version / Assigment Join Index

-- Drop old indexes
drop index if exists idx_assignment_version_join, idx_segment_version_join;

-- Create new indexes
create index if not exists idx_assignment_version_join_v2 on assignment (assignment_scope_urn, platform, distribution,
                                                                         architecture, status, segment_id, version_type,
                                                                         version_major, version_minor, version_patch,
                                                                         version_build, updated_at,
                                                                         segment_type) where status = 1;

create index if not exists idx_segment_version_join_v2 on segment_version (segment_id, platform, distribution,
                                                                           architecture, version_type, version_major,
                                                                           version_minor, version_patch, version_build,
                                                                           segment_data_id, effective_major,
                                                                           effective_minor, effective_patch,
                                                                           effective_build, updated_at);


--
-- Segment Data to Segment Version Trigger


-- Update existing values in segment_data

do
$do$
    begin
        -- Check if v2 of trigger exists, if not create the dependent functions
        if not exists(select tgname from pg_trigger where tgname = 'trigger_segment_versions_v2') then

            raise notice 'Creating function insert_segment_version_row';
            create or replace function insert_segment_version_row(new_data segment_data, version_type smallint) returns boolean AS
            $$
            declare
                PINNED       smallint := 0;
                NON_BREAKING smallint := 1;
                LATEST       smallint := 2;
                PATCH        smallint := 3; -- new
                BUILD        smallint := 4; -- new
                NA           smallint := -1;
            begin
                if version_type = PINNED then
                    insert into segment_version (segment_version_id, segment_data_id, segment_id, platform,
                                                 distribution, architecture,
                                                 version_type, version_major, version_minor, version_patch,
                                                 version_build, effective_major,
                                                 effective_minor, effective_patch, effective_build, created_at,
                                                 updated_at, url, content_hash, content_size)
                    values (default, new_data.segment_data_id, new_data.segment_id, new_data.platform,
                            new_data.distribution,
                            new_data.architecture, PINNED,
                            new_data.major, new_data.minor, new_data.patch, new_data.build, new_data.major,
                            new_data.minor, new_data.patch, new_data.build,
                            new_data.created_at, new_data.updated_at, new_data.url, new_data.content_hash,
                            new_data.content_size);
                elseif version_type = LATEST then
                    insert into segment_version (segment_version_id, segment_data_id, segment_id, platform,
                                                 distribution, architecture,
                                                 version_type, version_major, version_minor, version_patch,
                                                 version_build, effective_major,
                                                 effective_minor, effective_patch, effective_build, created_at,
                                                 updated_at, url, content_hash, content_size)
                    values (default, new_data.segment_data_id, new_data.segment_id, new_data.platform,
                            new_data.distribution,
                            new_data.architecture,
                            LATEST, NA, NA, NA, NA, new_data.major, new_data.minor, new_data.patch, new_data.build,
                            new_data.created_at, new_data.updated_at, new_data.url, new_data.content_hash,
                            new_data.content_size);

                elseif version_type = NON_BREAKING then
                    insert into segment_version (segment_version_id, segment_data_id, segment_id, platform,
                                                 distribution, architecture,
                                                 version_type, version_major, version_minor, version_patch,
                                                 version_build, effective_major,
                                                 effective_minor, effective_patch, effective_build, created_at,
                                                 updated_at, url, content_hash, content_size)
                    values (default, new_data.segment_data_id, new_data.segment_id, new_data.platform,
                            new_data.distribution,
                            new_data.architecture,
                            NON_BREAKING, new_data.major, NA, NA, NA, new_data.major, new_data.minor, new_data.patch,
                            new_data.build,
                            new_data.created_at, new_data.updated_at, new_data.url, new_data.content_hash,
                            new_data.content_size);
                elseif version_type = PATCH then
                    insert into segment_version (segment_version_id, segment_data_id, segment_id, platform,
                                                 distribution, architecture,
                                                 version_type, version_major, version_minor, version_patch,
                                                 version_build, effective_major,
                                                 effective_minor, effective_patch, effective_build, created_at,
                                                 updated_at, url, content_hash, content_size)
                    values (default, new_data.segment_data_id, new_data.segment_id, new_data.platform,
                            new_data.distribution,
                            new_data.architecture,
                            PATCH, new_data.major, new_data.minor, NA, NA, new_data.major, new_data.minor,
                            new_data.patch, new_data.build,
                            new_data.created_at, new_data.updated_at, new_data.url, new_data.content_hash,
                            new_data.content_size);
                elseif version_type = BUILD then
                    insert into segment_version (segment_version_id, segment_data_id, segment_id, platform,
                                                 distribution, architecture,
                                                 version_type, version_major, version_minor, version_patch,
                                                 version_build, effective_major,
                                                 effective_minor, effective_patch, effective_build, created_at,
                                                 updated_at, url, content_hash, content_size)
                    values (default, new_data.segment_data_id, new_data.segment_id, new_data.platform,
                            new_data.distribution,
                            new_data.architecture,
                            BUILD, new_data.major, new_data.minor, new_data.patch, NA, new_data.major, new_data.minor,
                            new_data.patch,
                            new_data.build,
                            new_data.created_at, new_data.updated_at, new_data.url, new_data.content_hash,
                            new_data.content_size);
                end if;
                return true;
            end;
            $$ language plpgsql;

            raise notice 'Creating function update_segment_version_row';
            create or replace function update_segment_version_row(new_data segment_data, version_type smallint,
                                                                  existing_row segment_version) returns boolean AS
            $$
            declare
                NON_BREAKING smallint := 1;
                LATEST       smallint := 2;
                PATCH        smallint := 3;
                BUILD        smallint := 4;
            begin
                if version_type = LATEST then
                    -- Update LATEST entry
                    UPDATE segment_version
                    SET segment_data_id = new_data.segment_data_id,
                        effective_major = new_data.major,
                        effective_minor = new_data.minor,
                        effective_patch = new_data.patch,
                        effective_build = new_data.build,
                        url             = new_data.url,
                        content_hash    = new_data.content_hash,
                        content_size    = new_data.content_size,
                        updated_at      = new_data.updated_at
                    where segment_version_id = existing_row.segment_version_id;
                elseif version_type = NON_BREAKING then
                    -- Update NON_BREAKING entry
                    UPDATE segment_version
                    SET segment_data_id = new_data.segment_data_id,
                        version_major   = new_data.major,
                        effective_major = new_data.major,
                        effective_minor = new_data.minor,
                        effective_patch = new_data.patch,
                        effective_build = new_data.build,
                        url             = new_data.url,
                        content_hash    = new_data.content_hash,
                        content_size    = new_data.content_size,
                        updated_at      = new_data.updated_at
                    where segment_version_id = existing_row.segment_version_id;
                elseif version_type = PATCH then
                    -- Update PATCH entry
                    UPDATE segment_version
                    SET segment_data_id = new_data.segment_data_id,
                        version_major   = new_data.major,
                        version_minor   = new_data.minor,
                        effective_major = new_data.major,
                        effective_minor = new_data.minor,
                        effective_patch = new_data.patch,
                        effective_build = new_data.build,
                        url             = new_data.url,
                        content_hash    = new_data.content_hash,
                        content_size    = new_data.content_size,
                        updated_at      = new_data.updated_at
                    where segment_version_id = existing_row.segment_version_id;
                elseif version_type = BUILD then
                    -- Update BUILD entry
                    UPDATE segment_version
                    SET segment_data_id = new_data.segment_data_id,
                        version_major   = new_data.major,
                        version_minor   = new_data.minor,
                        version_patch   = new_data.patch,
                        effective_major = new_data.major,
                        effective_minor = new_data.minor,
                        effective_patch = new_data.patch,
                        effective_build = new_data.build,
                        url             = new_data.url,
                        content_hash    = new_data.content_hash,
                        content_size    = new_data.content_size,
                        updated_at      = new_data.updated_at
                    where segment_version_id = existing_row.segment_version_id;
                end if;
                return true;
            end;
            $$ language plpgsql;

            raise notice 'Creating function to fill_content_hash_and_size';
            create or replace function fill_content_hash_and_size(sd_row segment_data) returns boolean as
            $$
            declare
                HASH   varchar(64) := encode(digest(sd_row.body, 'sha256'), 'hex');
                SIZE   bigint      := octet_length(sd_row.body);
            begin


                -- Update the segment data row
                update segment_data
                set content_hash = HASH,
                    content_size = SIZE
                where segment_data_id = sd_row.segment_data_id;

                -- Update all segment version rows

                update segment_version
                set content_hash = HASH,
                    content_size = SIZE
                where segment_data_id = sd_row.segment_data_id;

                return true;
            end;
            $$ language plpgsql;
        end if;
    end;
$do$;


-- Locks:
-- segment_data: EXCLUSIVE, prevent WRITEs to the table
--               This is prevent any new rows in the table while the trigger isn't active
-- segment_data: ACCESS EXCLUSIVE, prevent READs and WRITEs to the table
--               While "recreating" the rows of segment_version we need to ensure no one can access
--               the table. if the following operation takes too long, this will cause performance issues
--               in RMS.

begin WORK;
lock table segment_data in exclusive mode;
lock table segment_version in access exclusive mode;

do
$do$
    declare
        ROW_PINNED            record;
        ROW_PINNED_ASSIGNMENT record;
        ROW_SEGMENT_DATA      segment_data;
        ROW_SD_TO_FILL        segment_data;
        PINNED                smallint := 0;
        PATCH                 smallint := 3;
        BUILD                 smallint := 4;
    begin
        if not exists(select tgname from pg_trigger where tgname = 'trigger_segment_versions_v2') then

            raise notice 'Disabling trigger_segment_versions trigger';
            alter table if exists segment_data
                disable trigger trigger_segment_versions;

            raise notice 'Updating existing segment version rows ahead of new trigger creation';

            -- Check if there are existing PATCH or BUILD version_types in the table. If there are, this script
            -- is probably running twice, which it should never.
            if not exists((select * from segment_version where version_type = PATCH or version_type = BUILD)) then

                -- For each pinned row: insert a PATCH and BUILD row
                for ROW_PINNED in
                    select * from segment_version where version_type = PINNED
                    loop
                        select *
                        from segment_data
                        where segment_data_id = ROW_PINNED.segment_data_id
                        into ROW_SEGMENT_DATA;

                        perform insert_segment_version_row(ROW_SEGMENT_DATA, PATCH);
                        perform insert_segment_version_row(ROW_SEGMENT_DATA, BUILD);

                        update segment_version
                        set version_patch = 0,
                            version_build = 0
                        where segment_version_id = ROW_PINNED.segment_version_id;
                    end loop;

                -- For PINNED assignment rows set patch and build to 0
                for ROW_PINNED_ASSIGNMENT in
                    select * from assignment where version_type = PINNED
                    loop
                        update assignment
                        set version_patch = 0,
                            version_build = 0
                        where assignment_id = ROW_PINNED_ASSIGNMENT.assignment_id;
                    end loop;

                -- Fill all existing segment data and version rows with updated content hash and size
                for ROW_SD_TO_FILL in
                    select * from segment_data
                    loop
                        perform fill_content_hash_and_size(ROW_SD_TO_FILL);
                    end loop;
            else
                raise notice 'ERROR: Attempted to update existing segment versions, but failed. (no harm was done)';
                raise notice '    -  segment versions table already includes PATCH or BUILD version types. ';
                raise notice '    -  This is probably because the segment versions table has already been updated';
                raise notice '    -  before. This script should not run twice. The most likely cause it ran twice is that';
                raise notice '    -  the trigger_segment_versions_v2 trigger has been deleted.';
            end if;

            raise notice 'Creating segment data trigger version 2';

            create or replace function process_segment_versions_v2() returns trigger AS
            $trigger_segment_versions_v2$
            declare
                PINNED                                smallint := 0;
                NON_BREAKING                          smallint := 1;
                LATEST                                smallint := 2;
                PATCH                                 smallint := 3; -- new
                BUILD                                 smallint := 4; -- new

                NA                                    smallint := -1;
                EXISTING_BUILD_WITH_MAJOR_MINOR_PATCH segment_version%ROWTYPE; -- new
                EXISTING_PATCH_WITH_MAJOR_MINOR       segment_version%ROWTYPE; -- new
                EXISTING_NON_BREAKING_WITH_MAJOR      segment_version%ROWTYPE;
                EXISTING_LATEST                       segment_version%ROWTYPE;
                SEGMENT_DATA_EXISTS                   boolean  := (select exists
                                                                              (select 1
                                                                               FROM segment_data
                                                                               where segment_data.segment_id = NEW.segment_id));
            begin
                --
                -- Trigger to insert or update segment_version records based on new segment_data
                --
                if (TG_OP = 'INSERT') then
                    -- ALWAYS insert a new PINNED segment_version
                    perform insert_segment_version_row(NEW, PINNED);

                    if NOT SEGMENT_DATA_EXISTS then
                        -- Fast path if the segment data doesn't exist at all
                        perform insert_segment_version_row(NEW, LATEST);
                        perform insert_segment_version_row(NEW, NON_BREAKING);
                        perform insert_segment_version_row(NEW, PATCH);
                        perform insert_segment_version_row(NEW, BUILD);

                    else
                        -- BUILD
                        select *
                        into EXISTING_BUILD_WITH_MAJOR_MINOR_PATCH
                        FROM segment_version
                        where segment_id = NEW.segment_id
                          and version_type = BUILD
                          and effective_major = NEW.major
                          and effective_minor = NEW.minor
                          and effective_patch = NEW.patch
                          and platform = NEW.platform
                          and distribution = NEW.distribution
                          and architecture = NEW.architecture;

                        if (EXISTING_BUILD_WITH_MAJOR_MINOR_PATCH is null)
                        then
                            -- No BUILD ENTRY with this major, minor and patch version, inserting new entry
                            perform insert_segment_version_row(NEW, BUILD);
                        elseif (EXISTING_BUILD_WITH_MAJOR_MINOR_PATCH.effective_build < NEW.build)
                        then
                            -- Update BUILD
                            perform update_segment_version_row(NEW, BUILD, EXISTING_BUILD_WITH_MAJOR_MINOR_PATCH);
                        end if;

                        -- PATCH
                        select *
                        into EXISTING_PATCH_WITH_MAJOR_MINOR
                        FROM segment_version
                        where segment_id = NEW.segment_id
                          and version_type = PATCH
                          and effective_major = NEW.major
                          and effective_minor = NEW.minor
                          and platform = NEW.platform
                          and distribution = NEW.distribution
                          and architecture = NEW.architecture;

                        if (EXISTING_PATCH_WITH_MAJOR_MINOR is null)
                        then
                            -- No PATCH ENTRY with this major and minor version, inserting new entry
                            perform insert_segment_version_row(NEW, PATCH);
                        elseif (EXISTING_PATCH_WITH_MAJOR_MINOR.effective_patch < NEW.patch
                            OR (EXISTING_PATCH_WITH_MAJOR_MINOR.effective_patch = NEW.patch
                                and
                                EXISTING_PATCH_WITH_MAJOR_MINOR.effective_build < NEW.build))
                        then
                            -- Update PATCH
                            perform update_segment_version_row(NEW, PATCH, EXISTING_PATCH_WITH_MAJOR_MINOR);
                        end if;

                        -- Non BREAKING
                        select *
                        into EXISTING_NON_BREAKING_WITH_MAJOR
                        FROM segment_version
                        where segment_id = NEW.segment_id
                          and version_type = NON_BREAKING
                          and effective_major = NEW.major
                          and platform = NEW.platform
                          and distribution = NEW.distribution
                          and architecture = NEW.architecture;

                        if (EXISTING_NON_BREAKING_WITH_MAJOR is null)
                        then
                            -- No NON_BREAKING ENTRY with this major version, inserting new entry
                            perform insert_segment_version_row(NEW, NON_BREAKING);
                        elseif (EXISTING_NON_BREAKING_WITH_MAJOR.effective_minor < NEW.minor
                            OR (EXISTING_NON_BREAKING_WITH_MAJOR.effective_minor = NEW.minor
                                and
                                EXISTING_NON_BREAKING_WITH_MAJOR.effective_patch < NEW.patch)
                            OR (EXISTING_NON_BREAKING_WITH_MAJOR.effective_minor = NEW.minor
                                and
                                EXISTING_NON_BREAKING_WITH_MAJOR.effective_patch = NEW.patch
                                and
                                EXISTING_NON_BREAKING_WITH_MAJOR.effective_build < NEW.build))
                        then
                            -- Update NON_BREAKING
                            perform update_segment_version_row(NEW, NON_BREAKING, EXISTING_NON_BREAKING_WITH_MAJOR);
                        end if;

                        -- LATEST
                        select *
                        into EXISTING_LATEST
                        FROM segment_version
                        where segment_id = NEW.segment_id
                          and version_type = LATEST
                          and platform = NEW.platform
                          and distribution = NEW.distribution
                          and architecture = NEW.architecture;

                        if (EXISTING_LATEST is null) then
                            -- No LATEST entry at all, inserting new entry
                            perform insert_segment_version_row(NEW, LATEST);
                        elseif (EXISTING_LATEST.effective_major < NEW.major
                            OR (EXISTING_LATEST.effective_major = NEW.major
                                and
                                EXISTING_LATEST.effective_minor < NEW.minor)
                            OR (EXISTING_LATEST.effective_major = NEW.major
                                and
                                EXISTING_LATEST.effective_minor = NEW.minor
                                and
                                EXISTING_LATEST.effective_patch < NEW.patch)
                            OR (EXISTING_LATEST.effective_major = NEW.major
                                and
                                EXISTING_LATEST.effective_minor = NEW.minor
                                and
                                EXISTING_LATEST.effective_patch = NEW.patch
                                and
                                EXISTING_LATEST.effective_build < NEW.build)
                            ) then
                            -- Update LATEST
                            perform update_segment_version_row(NEW, LATEST, EXISTING_LATEST);
                        end if;
                    end if;
                end if;
                return null; -- result is ignored since this is an AFTER trigger
            end;
            $trigger_segment_versions_v2$ language plpgsql;

            create trigger trigger_segment_versions_v2
                after insert
                on segment_data
                for each row
            execute procedure process_segment_versions_v2();
        else
            raise notice 'Segment Version Trigger V2 already exists, skipping.';
        end if;
    end
$do$;

commit work;
