--
-- Version Tagging Phase 2
-- org_version_tag table with fields: org_key and version_tag
-- segment_class field in segment_type table
--

-- org_version_tag table schema
CREATE TABLE IF NOT EXISTS org_version_tag (
    org_urn varchar(24) not null,
    version_tag varchar(64) not null,
    PRIMARY KEY (org_urn, version_tag),
    FOREIGN KEY (version_tag) REFERENCES named_versions(version_tag) ON DELETE CASCADE,
    UNIQUE (org_urn)
);

-- Schema update segment_type
-- Default is a SegmentClass of NONE which maps to smallint 0
alter table if exists segment_type
    add column if not exists segment_class smallint not null default 0;

-- Add PRULES SegmentClass and TAGGED version types to all product rules

update segment_type
set
segment_class=1,
version_types = 'NON_BREAKING,PATCH,BUILD,TAGGED',
filter_strategies = 'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION'
where name = 'DC_REPORT';

update segment_type
set
    segment_class=1,
    version_types = 'NON_BREAKING,PATCH,BUILD,TAGGED',
    filter_strategies = 'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION'
where name = 'DC_BLOCK';

update segment_type
set
    segment_class=1,
    version_types = 'NON_BREAKING,PATCH,BUILD,TAGGED',
    filter_strategies = 'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION'
where name = 'AMSI_TI';

update segment_type
set
    segment_class=1,
    version_types = 'NON_BREAKING,PATCH,BUILD,TAGGED',
    filter_strategies = 'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION'
where name = 'AMSI_ASP';

update segment_type
set
    segment_class=1,
    version_types = 'PINNED,NON_BREAKING,PATCH,BUILD,TAGGED',
    filter_strategies = 'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION'
where name = 'TH_BASE';

update segment_type
set
    segment_class=1,
    version_types = 'NON_BREAKING,PATCH,BUILD,TAGGED',
    filter_strategies = 'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION'
where name = 'SCRIPT_REPORT';

update segment_type
set
    segment_class=1,
    version_types = 'NON_BREAKING,PATCH,BUILD,TAGGED',
    filter_strategies = 'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION'
where name = 'TAMPER';

update segment_type
set
    segment_class=1,
    version_types = 'PINNED,NON_BREAKING,PATCH,BUILD,TAGGED',
    filter_strategies = 'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION'
where name = 'UBS';

update segment_type
set
    segment_class=1,
    version_types = 'PINNED,NON_BREAKING,PATCH,BUILD,TAGGED',
    filter_strategies = 'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION'
where name = 'FILE_UPLOAD_TRIGGER';

update segment_type
set
    segment_class=1,
    version_types = 'NON_BREAKING,PATCH,BUILD,TAGGED',
    filter_strategies = 'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION'
where name = 'SENSOR_TELEMETRY';

-- Add net new segment types for DSER-27105

-- HASHBAN
insert into segment_type (segment_type_id, segment_tag, name, platforms, packaging, version_scheme,
                          version_types, segment_scopes, assign_scopes, allow_app, filter_strategies, segment_class)
values (17, 0, 'HASHBAN', 'WINDOWS', 2, 2, 'NON_BREAKING,PATCH,BUILD,TAGGED', 'REGION,ORG', 'REGION,ORG,POLICY', false,
        'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION', 1)
on conflict do nothing;
-- HASHBAN
update segment_type
set segment_type_id=17, segment_tag=0, name='HASHBAN', platforms='WINDOWS', packaging=2, version_scheme=2,
    version_types='NON_BREAKING,PATCH,BUILD,TAGGED', segment_scopes='REGION,ORG', assign_scopes='REGION,ORG,POLICY', allow_app=false,
    filter_strategies='DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION', segment_class=1
where segment_type_id=17;

-- APPHOOK
insert into segment_type (segment_type_id, segment_tag, name, platforms, packaging, version_scheme,
                          version_types, segment_scopes, assign_scopes, allow_app, filter_strategies, segment_class)
values (18, 10, 'APPHOOK', 'WINDOWS', 2, 2, 'NON_BREAKING,PATCH,BUILD,TAGGED', 'REGION', 'REGION,ORG', false,
        'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION', 1)
on conflict do nothing;
-- APPHOOK
update segment_type
set segment_type_id=18, segment_tag=10, name='APPHOOK', platforms='WINDOWS', packaging=2, version_scheme=2,
    version_types='NON_BREAKING,PATCH,BUILD,TAGGED', segment_scopes='REGION', assign_scopes='REGION,ORG', allow_app=false,
    filter_strategies='DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION', segment_class=1
where segment_type_id=18;

-- CBD
insert into segment_type (segment_type_id, segment_tag, name, platforms, packaging, version_scheme,
                          version_types, segment_scopes, assign_scopes, allow_app, filter_strategies, segment_class)
values (19, 10, 'CBD', 'WINDOWS', 2, 2, 'NON_BREAKING,PATCH,BUILD,TAGGED', 'REGION', 'REGION,ORG,POLICY', false,
        'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION', 1)
on conflict do nothing;
-- CBD
update segment_type
set segment_type_id=19, segment_tag=10, name='CBD', platforms='WINDOWS', packaging=2, version_scheme=2,
    version_types='NON_BREAKING,PATCH,BUILD,TAGGED', segment_scopes='REGION', assign_scopes='REGION,ORG,POLICY', allow_app=false,
    filter_strategies='DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION', segment_class=1
where segment_type_id=19;

-- TELEMETRY
insert into segment_type (segment_type_id, segment_tag, name, platforms, packaging, version_scheme,
                          version_types, segment_scopes, assign_scopes, allow_app, filter_strategies, segment_class)
values (20, 11, 'TELEMETRY', 'WINDOWS', 2, 2, 'NON_BREAKING,PATCH,BUILD,TAGGED', 'REGION', 'REGION,ORG', false,
        'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION', 1)
on conflict do nothing;
-- TELEMETRY
update segment_type
set segment_type_id=20, segment_tag=11, name='TELEMETRY', platforms='WINDOWS', packaging=2, version_scheme=2,
    version_types='NON_BREAKING,PATCH,BUILD,TAGGED', segment_scopes='REGION', assign_scopes='REGION,ORG', allow_app=false,
    filter_strategies='DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION', segment_class=1
where segment_type_id=20;

-- RANSOMWARE
insert into segment_type (segment_type_id, segment_tag, name, platforms, packaging, version_scheme,
                          version_types, segment_scopes, assign_scopes, allow_app, filter_strategies, segment_class)
values (21, 10, 'RANSOMWARE', 'WINDOWS', 2, 2, 'NON_BREAKING,PATCH,BUILD,TAGGED', 'REGION', 'REGION,ORG', false,
        'DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION', 1)
on conflict do nothing;
-- RANSOMWARE
update segment_type
set segment_type_id=21, segment_tag=10, name='RANSOMWARE', platforms='WINDOWS', packaging=2, version_scheme=2,
    version_types='NON_BREAKING,PATCH,BUILD,TAGGED', segment_scopes='REGION', assign_scopes='REGION,ORG', allow_app=false,
    filter_strategies='DEDUP_SEGMENT_DATA,MOST_EXPLICIT_SCOPE,HIGHEST_SEMANTIC_VERSION', segment_class=1
where segment_type_id=21;
