
-- A function that will identify segment data records of type UDS_BUNDLE and whose creation date is older
-- than 30 minutes and delete them.
CREATE OR REPLACE FUNCTION delete_old_segment_data_of_type_uds_bundle() RETURNS trigger AS
$$
DECLARE
 row_count int;
BEGIN
  DELETE FROM segment_data sd
  WHERE sd.segment_data_id IN
  (
      SELECT sd.segment_data_id
      FROM segment_data sd
           JOIN segment s ON sd.segment_id = s.segment_id
           JOIN segment_type st ON st.segment_type_id = s.segment_type
      WHERE st.name IN ('UDS_BUNDLE') AND s.segment_id = new.segment_id
      AND sd.created_at < EXTRACT(EPOCH FROM NOW() - INTERVAL '30 minutes')*1000
  );
  IF found THEN
    GET DIAGNOSTICS row_count = ROW_COUNT;
    RAISE NOTICE 'DELETED % row(s) of type UDS_BUNDLE FROM segment_data', row_count;
  END IF;
  RETURN NEW;
END;
$$
LANGUAGE 'plpgsql';

-- Drop the trigger if it already exists --
DROP TRIGGER IF EXISTS trigger_delete_old_segment_data_of_type_uds_bundle ON segment_data;

-- Create the trigger  --
CREATE TRIGGER trigger_delete_old_segment_data_of_type_uds_bundle
BEFORE INSERT ON segment_data
FOR EACH ROW
EXECUTE PROCEDURE delete_old_segment_data_of_type_uds_bundle();